/**
 * @file include/histdataheff.hpp
 *
 * @brief  Read a *.dsp binary file generated by the HEFF code
 *
 * @author Jordan Bieder <jordan.bieder@cea.fr>
 *
 * @copyright Copyright 2014 Jordan Bieder
 *
 * This file is part of Agate.
 *
 * Agate is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Agate is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Agate.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef HISTDATAHEFF_HPP
#define HISTDATAHEFF_HPP

#ifdef _WIN32
#include "base/win32.hpp"
#endif

#ifdef HAVE_CONFIG_H
#include "agate.h"
#undef HAVE_CONFIG_H
#endif

#include "hist/histdata.hpp"

/** 
 * handle a *.dsp file in binary created by the Heff code.
 * Contains only the displacements and strain
 */
class HistDataHeff : public HistData {

  private :

  protected :

  public :

    /**
     * Constructor.
     */
    HistDataHeff();

    /**
     * Destructor.
     */
    virtual ~HistDataHeff();

    /**
     * Open a file and read it to fill the data
     * @param filename Name of the file
     */
    virtual void readFromFile(const std::string& filename);
};

#endif  // HISTDATAHEFF_HPP
